#!/bin/sh

ORGDIR="`pwd`"
BUCKET="s3://project-roots"
ID="$1"

if [ `pgrep "s3put" | wc -l` -gt 2 ]; then 
    echo `pgrep "s3put" | wc -l`;
    echo "another s3 instance is already running :/";
    exit 1;
fi


echo "LOCAL <- $ID"


echo "creating tmp dir.."
DIR=/tmp/s3/`cat /proc/sys/kernel/random/uuid`

mkdir -p $DIR;

cd $DIR;


echo "initiating downloads.."
PIDS="";
for i in `s3cmd ls $BUCKET/${ID}_ | grep -Eo ${ID}.*$`; do 
    echo $i; 
    s3cmd get "$BUCKET/$i" $i > /dev/null &
    PIDS="$PIDS $!";
done

for pid in $PIDS; do
    echo "waiting for process $pid"; 
    wait $pid;
done

echo "verifying checksums..";
sha1sum -c *.sha1 || exit 1

rm *.sha1;

echo "joining..";
cat ${ID}_* > $ID.gz ; 
rm ${ID}_*

echo "unzipping.."
gunzip $ID.gz

echo "verifying final checksum..";
s3cmd get "$BUCKET/$ID.sha1" $ID.sha1;
sha1sum -c $ID.sha1 || exit 1

echo "moving ..";
mv $ID "$ORGDIR/"
rm $ID.sha1;

rm -vrf $DIR;

echo "done.";

