#!/bin/sh

BUCKET="s3://project-roots"
ID="$1"
FILE="$2"

if [ `pgrep "s3put" | wc -l` -gt 2 ]; then 
    echo `pgrep "s3put" | wc -l`
    echo "another s3 instance is already running :(";
    exit 1;
fi


echo "$ID <- $FILE"


echo "remove old files.."

DIR=/tmp/s3/`cat /proc/sys/kernel/random/uuid`
mkdir -p $DIR


echo "copying.."
cp $FILE $DIR/$ID

cd $DIR


echo "hashing.."
sha1sum $ID > $ID.sha1 ;
s3cmd put $ID.sha1 "$BUCKET/$ID.sha1"

echo "gzipping.."
gzip $ID
FILE=$ID.gz

echo "splitting.."
split --bytes 16MB $FILE ${ID}_

echo "initiating uploads.."
PIDS="";
for i in ${ID}_*; do 
    echo $i; 
    sha1sum $i > $i.sha1 ;
    s3cmd put $i.sha1 "$BUCKET/$i.sha1" > /dev/null;
    s3cmd put $i      "$BUCKET/$i"      > /dev/null &
    PIDS="$PIDS $!"
done

for pid in $PIDS; do
    echo "waiting for process $pid"; 
    wait $pid;
done

echo "cleaning up.."
rm -vfr $DIR