#include <python2.6/Python.h>

#include <stdio.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <errno.h>
#include <string.h>
#include <math.h>
#include <assert.h>

#include <gmp.h>

#include "ffc.h"


#include "freqs.c"
#include "chi2.c"
//#include "ks.c"



#define UPDATE_WW(prev_run, digit, runs, nrun, prun)    \
  if(digit <= med_n) {                                  \
    nrun += 1;                                          \
    if(prev_run != -1) {                                \
      prev_run = -1;                                    \
      runs += 1;                                        \
    }                                                   \
  } else if (digit >= med_p) {                          \
    prun += 1;                                          \
    if(prev_run != 1) {                                 \
      prev_run = 1;                                     \
      runs += 1;                                        \
    }                                                   \
  }                                                     \


int ww_is_ok(runs, nval, pval) {
  int retval = 0;
  unsigned long N = nval + pval;
  // u = ((2 * pval * nval) / N) + 1
  mpf_t u;
  mpf_init_set_ui(u, 2 * nval);
  mpf_mul_ui(u, u, pval);
  mpf_div_ui(u, u, N);
  mpf_add_ui(u, u, 1);
  // r2 = ((u-1)*(u-2))/(N-1)
  mpf_t tmp0, tmp1;
  mpf_init_set(tmp0, u);
  mpf_init_set(tmp1, u);
  mpf_sub_ui(tmp0, u, 1);
  mpf_sub_ui(tmp1, u, 2);
  mpf_t r2;
  mpf_init_set(r2, tmp0);
  mpf_mul(r2, r2, tmp1);
  mpf_div_ui(r2, r2, N-1);
  // r = sqrt(r2)
  mpf_t r;
  mpf_init(r);
  mpf_sqrt(r, r2);
  // reg = 1.96 * r
  mpf_t reg, c;
  mpf_init_set(reg, r);
  mpf_init_set_d(c, 1.96);
  mpf_mul(reg, reg, c);
  printf("reg: %s\nu: %s\nruns: %ld\n", mpf_to_str(reg), mpf_to_str(u), runs);
  // - reg < runs - u < reg => retval = 1
  mpf_neg(u, u);
  mpf_add_ui(u, u, runs);
  mpf_abs(u, u);
  if (mpf_cmp(reg, u) == 1) {
    retval = 1;
  }
  // clean up
  mpf_clear(u);
  mpf_clear(tmp0);
  mpf_clear(tmp1);
  mpf_clear(r2);
  mpf_clear(r);
  mpf_clear(reg);
  mpf_clear(c);
  return retval;
}


char* mpf_to_str(mpf_t f) {
	char* fs;
	gmp_asprintf(&fs, "%.*Ff", MPF_PREC/4, f);
	return fs;
}

static PyObject*
count_freqs(const char buffer[], int len, size_t base,
	    size_t runlength, size_t block_size) {
  // the Python list to return
  PyObject* py_list = PyList_New(0);

  // init counters
  mpz_t moutcomes;
  mpz_init_set_ui(moutcomes, base);
  mpz_pow_ui(moutcomes, moutcomes, runlength);
  int outcomes = mpz_get_ui(moutcomes);
  mpz_clear(moutcomes);

  unsigned int* counts  = calloc(outcomes, sizeof(int));
	
  // read first value as string
  char block[runlength+1];
  block[runlength] = 0; // \0 terminated string
  memcpy(block, buffer, runlength);

  // convert first value to long
  mpz_t tmp;
  mpz_init(tmp);
  mpz_set_str(tmp, block, base);

  int digit  = mpz_get_ui(tmp);
  mpz_clear(tmp);

  int previous[block_size];
  long i,j;
  for(i = 0; i < runlength; i++) {
    previous[i] = block[i]-48;
  }
  // reinit block to zeros
  memset(block, 0, runlength);
  
  int factor = outcomes / base;
  
  unsigned long o2=0;
  
  clock_t start,end;
  
  char *freq=NULL, *chi2=NULL, *wwO, *wwS; //, *ks;
  
  long unsigned min = 0;
  long unsigned min_idx = 0;
  long unsigned max = 0;

  unsigned long runsO = 0;
  unsigned long prunO = 0;
  unsigned long nrunO = 0;
  unsigned long runsS = 0;
  unsigned long prunS = 0;
  unsigned long nrunS = 0;
  int prev_digitO = 0;
  int prev_digitS = 0;
  int rS = runlength-1;

  int med_n = outcomes/2-1;
  int med_p = outcomes/2+(outcomes%2);

  mpf_t qi;
  mpf_init(qi);
  printf("counting..\n");
  int p = runlength;
  start = clock();
  for(i = runlength; i < len; i++,p++) {
    // increment value
    assert (digit >= 0);
    unsigned long prev = counts[digit];
    unsigned long next = prev + 1;
    counts[digit] = next;
    
    max = next > max ? next : max;
    
    // update o2
    o2 -= prev*prev;
    o2 += next*next;
    
    // update digit
    assert(buffer[i] >= 48 && buffer[i] <= 48 + base);
    int new = buffer[i]-48;
    //digit = DIGIT_MOVE(digit, (buffer[i]-48));
    digit = (digit - previous[0]*factor)*base + new;
    //printf("%d\n", previous[0]);
    int q;
    for(q = 1; q < runlength; q++) {
      previous[q-1] = previous[q];
    }
    previous[runlength-1] = new;
    // need to compute distributions?
    
    // update run counts if new run
    UPDATE_WW(prev_digitO, digit, runsO, nrunO, prunO);
    if (++rS == runlength) {
      rS = 0;
      UPDATE_WW(prev_digitS, digit, runsS, nrunS, prunS);
    }

    if (p < block_size) {
      continue;
    }
    p = 0;

    // make circulized
    unsigned long old_prev=prev_digitO;
    unsigned long old_runs=runsO, old_prun=prunO, old_nrun=nrunO;
    unsigned int cir_digit = digit;
    for(j = 0; j < runlength-1; j++) {
      cir_digit = DIGIT_MOVE(cir_digit, buffer[j]-48);
      assert (cir_digit >= 0 && cir_digit < outcomes);
      // update counts
      prev = counts[cir_digit];
      counts[cir_digit] = prev + 1;
      // update o2
      o2 -= prev*prev;
      o2 += (prev+1)*(prev+1);
      //o2 = o2 + 2*prev + 1;
      UPDATE_WW(prev_digitO, cir_digit, runsO, nrunO, prunO);
    }

    if(counts[min_idx] != min) {
      unsigned long old_min = min;
      min = len;
      for(j = 0; j < outcomes; j++) {
	if(counts[j] < min) {
          min = counts[j];
          min_idx = j;
          if(min == old_min) break;
	}
      }
    }
    // compute distributions
    freq = get_freqs(counts, i, base, runlength, outcomes, min, max);
    chi2 = get_chi2(counts, i, base, runlength, outcomes, o2);
    //ks   = get_ks(counts, i, base, runlength, outcomes);
    asprintf(&wwO, "%ld|%ld|%ld", runsO, nrunO, prunO);
    asprintf(&wwS, "%ld|%ld|%ld", runsS, nrunS, prunS);
    //printf("%s\n", wwO);
    //printf("%s\n", wwS);


    // undo circulized
    cir_digit = digit;
    for(j = 0; j < runlength-1; j++) {
      cir_digit = DIGIT_MOVE(cir_digit, buffer[j]-48);
      // update counts
      prev = counts[cir_digit];
      counts[cir_digit] = prev - 1;
      // update o2
      o2 -= prev*prev;
      o2 += (prev-1)*(prev-1);
    }

    runsO = old_runs;
    nrunO = old_nrun;
    prunO = old_prun;
    prev_digitO = old_prev;

    // add computed values to list as tuple
    PyObject* tuple = Py_BuildValue("{s:i,s:s,s:s,s:s,s:s}",
                                    "length", i,
                                    "freq", freq,
                                    "chi2", chi2,
                                    "wwO", wwO, "wwS", wwS);
    PyList_Append(py_list, tuple);
    }
	end = clock();
	printf("time used total: %ld\n", (end-start));
	printf("computing..\n");

	printf("freq:    %s\n", freq);
	printf("chi2:    %s\n", chi2);
	printf("wwO:     %s\n", wwO);
	printf("wwS:     %s\n", wwS);

	free(counts);
	return py_list;
}


static PyObject*
frequencies_count(PyObject *self, PyObject *args)
{
	const char* string;

	int base, runlength, block_size;

	if (!PyArg_ParseTuple(args, "siii", &string, &base,
			      &runlength, &block_size))
		return NULL;


	return count_freqs(string, strlen(string), base,
			   runlength, block_size);
}

PyMethodDef methods[] = {
    {"count", frequencies_count, METH_VARARGS,
     "Count frequencies in string"},
    {NULL, NULL, 0, NULL}
};

PyMODINIT_FUNC
init_ffc(void)
{
	mpf_set_default_prec(MPF_PREC);
	Py_InitModule("_ffc", methods);
}



int main()
{
	Py_Initialize();
	mpf_set_default_prec(MPF_PREC);

	printf("%d\n", ww_is_ok(386320000, 386363657, 386332058));
	return 0;

	char *fname = "pisot-6-b2-cut";

	struct stat buf;
	int ret;
	ret = stat(fname, &buf);
	if(ret) {
	  printf("could not open %s.\n", fname);
	  return 1;
	}
	printf("%ld\n", buf.st_size);
	char *s = malloc(buf.st_size+1);
	s[buf.st_size] = 0;
	if(!s) {
	  printf("Could not allocate buffer for reading.\n");
	  return 1;
	}

	FILE *fd = fopen(fname, "r");
	fread(s, buf.st_size, 1, fd);

	size_t N = buf.st_size;
	int block_size = 6053; //N / (2<<10);

	count_freqs(s, N, 2, 2, block_size);
	//count_freqs(s, 128, 2, 1, block_size);
	return 0;
}


