#include "ffc.h"

#include <gmp.h>


/**
 * \sum (e-o)^2 / e
 * \sum e + o^2/e - 2o
 * \sum e + \sum o^2/e - \sum 2o
 * Oe + \sum o^2/e - 2 \sum o
 * Oe + \sum o^2/e - 2N
 */


char* get_chi2(unsigned int* counts, size_t length,
	       int base, int block_size, unsigned int outcomes,
	       unsigned long o2) {

  mpf_t acc, expect;

  mpf_init_set_ui(expect, length);
  mpf_div_ui(expect, expect, outcomes);

  mpf_t sum,term;
  mpf_init(term);
  mpf_init_set_ui(sum, 0);
  unsigned long i = 0;
  unsigned long o = 0;
  unsigned long o2b = 0;
  for(i = 0; i < outcomes; i++) {
    unsigned long val = counts[i];
    mpf_set_ui(term, val);
    mpf_sub(term, term, expect);
    mpf_pow_ui(term, term, 2);
    mpf_add(sum, sum, term);
    o += val;
    o2b += val*val;
  }
  mpf_clear(term);

  mpf_div(sum, sum, expect);
  char* str_old = mpf_to_str(sum);
  mpf_clear(sum);
  

  
  unsigned long N = length;
  
  mpf_t neg;
  mpf_init_set_ui(neg, N*2);

  mpf_init(acc);
  mpf_mul_ui(acc, expect, outcomes);

  mpf_t add;
  mpf_init_set_ui(add, o2);
  mpf_div(add, add, expect);
  mpf_add(acc, acc, add);
  mpf_sub(acc, acc, neg);

  char* str_res = mpf_to_str(acc);


  mpf_set_ui(neg, N*2);
  mpf_mul(neg, neg, expect);

  mpf_set_ui(acc, 0);
  mpf_mul(acc, expect, expect);
  mpf_mul_ui(acc, acc, outcomes);

  mpf_add_ui(acc, acc, o2);

  mpf_sub(acc, acc, neg);
  mpf_div(acc, acc, expect);

  char* str_new = mpf_to_str(acc);

  //printf("%ld\n%ld\n", o2, o2b);
  assert(o2 == o2b);
  //printf("%s\n%s\n%s\n", str_res, str_new, str_old);
  assert(o == N);
  assert((mpf_sgn(acc)) != -1);
  
  mpf_clear(acc);
  mpf_clear(expect);
  mpf_clear(neg);
  mpf_clear(add);

  free(str_new);
  free(str_old);


  return str_res;
  /**
  int i;
  mpf_t val;
  mpf_init(val);

  for(i = 0; i < outcomes; i++) {
    //mpf_init_set_ui(val, counts[i]);
    mpf_sub_ui(val, expect, counts[i]);
    //mpf_sub(val, val, expect);
    mpf_pow_ui(val, val, 2);
    mpf_add(acc, acc, val);
  }
  mpf_clear(val);
  
  mpf_div(acc, acc, expect);
  
  char* str_res = mpf_to_str(acc);
  
  mpf_clear(acc);
  mpf_clear(expect);

  return str_res;
  **/
}

