#include <Python.h>

#include <math.h>
#include <string.h>
#include <gmp.h>



static PyObject * 
_freqs_count(PyObject *self, PyObject *args)
{
  const char *DATA;
  long *BLOCKI;
  long  *RUNLENGTH;
  long  *BASE;
  long *BLOCK_SIZE;

  if (!PyArg_ParseTuple(args, "sllll", &DATA, &BLOCKI, &RUNLENGTH, &BASE, &BLOCK_SIZE))
    return NULL;

  if (((long)BLOCKI & (long)15) == 0) {
    printf("%li\n", (long)BLOCKI);
  }

  const int N      = strlen(DATA);
  const int combs  = pow((long)BASE, (long)RUNLENGTH);
  const long block = (long)BLOCKI * (long)BLOCK_SIZE;

  const long from = block == 0 ? 0 : block - (long)RUNLENGTH+1;
  const long to   = block + (long)BLOCK_SIZE-(long)RUNLENGTH+1;

  // count frequencies
  int i; 
  char *buf;
  long *counts; 

  buf    = (char*) calloc((long)RUNLENGTH, sizeof(char));
  counts = (long*) calloc(combs, sizeof(long));

  // count digits
  unsigned long d_lng; 
  mpz_t *d_mpz = (mpz_t*)malloc(sizeof(mpz_t));
  mpz_init(*d_mpz);

  for(i = from; i < to; i++) {
    strncpy(buf, DATA + i, (long)RUNLENGTH);
    mpz_set_str(*d_mpz, buf, (long) BASE);
    d_lng = mpz_get_ui(*d_mpz);
    counts[d_lng]++;
  }
  free(buf);
  free(d_mpz);

  // build dictionary
  PyObject *dict, *key, *value;
  dict = PyDict_New();
  for(i = 0; i < combs; i++) {
    long value = counts[i];
    if (value == 0) {
      continue;
    }
    key = Py_BuildValue("i", i);
    PyDict_SetItem(dict, key, Py_BuildValue("l", value));
    Py_DECREF(key);
  }
  free(counts);
  //Py_DECREF(dict);

  return dict;

}


static PyMethodDef _FreqsMethods[] = {
  {"count", _freqs_count, METH_VARARGS, "count frequencies in string"}, 
  {NULL, NULL, 0, NULL} /* sentinel */
};


PyMODINIT_FUNC
init_freqs(void) 
{
  (void) Py_InitModule("_freqs", _FreqsMethods); 
}
