from timings import time_func
from roots import mp, nt, lm, pw, gm_old
import time, sys, math
from database import * 


pol_name = sys.argv[1]
N        = int(sys.argv[2])

pol = Polynomial().query_one('SELECT * FROM Polynomial WHERE name = %s', (pol_name,)).polynomial


log3 = lambda x : math.log(x) / math.log(3)



''' Name                Function                        When to skip? N> ''' 
order = ('GM', 'MP', 'PM', 'NT', 'LM')
methods = dict((
    ('GM',       (time_func(gm_old.gupta_mittal),  16)),
    ('MP',       (time_func(mp.mp_solve),          23)),
#    ('Gupta-Mittal',       gm.gupta_mittal,    100),
    ('PM',       (time_func(pw.power_method),      100)), # PM is always included (special case)
    ('NT',       (time_func(nt.newtons_method),    100)),
    ('LM',       (time_func(lm.laguerres_method),  100))
    ))

# power method is special case
print '> PM'
(cpu, (P, _)) = methods['PM'][0](pol, N)
pw_secs = cpu
pw_prec = P
pw_rate = P / cpu
print '%s rate: %8.2f' % ('PM', pw_rate)


results = []
for name in order:
    method, _max = methods[name]
    # skip if N is higher than limit
    if N > _max:
        results.append((name, '??', None, None))
        continue

    # power method was special case
    if name == 'PM':
        results.append((name, pw_prec, pw_rate, pw_secs))
        continue

    print '>', name

    # laguerre needs special care
    if name == 'LM':
        _N = int(math.ceil(log3(2**(N-1)))+1)
        secs, (P, root) = method(pol, _N)
    else:
        secs, (P, root) = method(pol, N)

    # gm has same precision as power method
    if method == gm_old.gupta_mittal:
        P = pw_prec

    # compute stats
    print secs
    rate = P / secs
    results.append((name, P, rate, secs))
    print '%s rate: %8.2f' % (name, rate)


for name, P, rate, secs in results:
    print '%32s %12s %12.2f %6.2f' % (name, P, rate or 0, secs or 0)


used = map(lambda x: (x[2] != None and  '%6.2f' % (x[2]/pw_rate)) or '??', results)
secs = map(lambda x: (x[3] != None and '%12.2f' % x[3]) or '??', results)
lst = zip(used, secs)

line = ('%4i & ' % N) + ' & '.join('%s & %s' % e for e in lst)

print line
file('tables/speed-%s.tex' % pol_name, 'a').write(line+r'\\'+'\n')

