from config import * 

import os.path
import models
import psycopg2 as dbapi2

# make sure dbapi matches implementation
assert dbapi2.apilevel == '2.0'
assert dbapi2.paramstyle == 'pyformat'

# import models
from models.polynomial      import Polynomial
from models.job             import Job
from models.frequency       import *
from models.distributions   import *

# custom row factory for dictionary based access
import psycopg2.extras
DICT_CURSOR = psycopg2.extras.DictCursor

# time functions
from psycopg2 import TimestampFromTicks as _TimestampFromTicks, DateFromTicks

# wrap around TimestampFromTicks to fix rounding error bug
TimestampFromTicks = lambda ticks: _TimestampFromTicks(int(ticks))


TABLES = ( Polynomial, Job, FrequencyGroup, FrequencyBlock, Distribution, DistributionResult )


CONNECTION = None

def db_connect():
    global CONNECTION
    if CONNECTION: 
        return CONNECTION
    # if no db, create it
    try:
        CONNECTION = dbapi2.connect(user=DB_USER, password=DB_PASS, database=DB_DATABASE, host=DB_HOST)
    except:
        conn = dbapi2.connect(user=DB_USER, password=DB_PASS, host=DB_HOST)
        conn.set_isolation_level(0)
        conn.cursor().execute('CREATE DATABASE %s' % DB_DATABASE)
        conn.close()
        CONNECTION = dbapi2.connect(user=DB_USER, password=DB_PASS, database=DB_DATABASE, host=DB_HOST)
        db_create()
    return CONNECTION

def db_cursor():
    db_connect()
    global CONNECTION
    return CONNECTION.cursor(cursor_factory=DICT_CURSOR)

def db_create():
    for T in TABLES:
        T().create()
        
def db_execute(sql, params=()):
    cursor = db_cursor()
    assert type(params) == tuple, ('db_execute: Expected tuple, got: %s' % params)
    params = params
    cursor.execute(sql, params)
    return cursor

def db_execute_one(sql, params=()):
    return db_execute(sql, params).fetchone()

AUTO_COMMIT=True
def db_commit(auto_commit=None):
    global AUTO_COMMIT
    if auto_commit != None:
        AUTO_COMMIT = auto_commit
    if AUTO_COMMIT:
        db_connect().commit()

def db_update(sql, params=()):
    db_execute(sql, params)
    db_commit()

def db_close():
    db_commit()
    db_connect().close()

