import matplotlib
matplotlib.use("Agg")


from pylab import * 

class Plot:
    
    def __init__(self):
        figure(figsize=(12,8))
        self.width = 1.0
        self.plot = plot
    
    def set_width(self, width):
        self.width = width

    def bar(self, heights, color='blue'):
        xs = tuple(i * self.width for i in xrange(len(heights)))
        return [ bar(x,h,width=self.width,color=color) for x,h in zip(xs, heights) ]

    def color_bar(self, heights, colors):
        xs = tuple(i * self.width for i in xrange(len(heights)))
        for (x, h), c in zip(zip(xs, heights), colors):
            bar(x, h, width=self.width, color=c)

    def error_bar(self, heights, errors):
        xs = tuple(i * self.width for i in xrange(len(heights)))

        return [ errorbar(x,h,error, fmt='x') 
                 for (x,h),error in zip(zip(xs, heights), errors) ]

    def flip_bar(self, xs, values, _colors=['blue','red']):
        _colors = list(_colors)
        labels = self.labels(xs)
        prev   = None
        colors = []
        color  = 0
        for x, value in zip(xs, values):
            if prev == None or x != prev:
                color = (color + 1) % len(_colors)
                prev = x
            colors.append(_colors[color])
        self.color_bar(values, colors)

    def labels(self, in_labels, valign='center'):
        labels, prev = [], None
        for label in in_labels:
            if prev != None and label == prev:
                labels.append('')
                continue
            labels.append(label)
            prev = label
        xs = tuple(i*self.width for i in xrange(len(labels)))
        xticks( xs, labels, verticalalignment=valign )
        return labels

    def linear(self, x_from, x_to, a, b):
        xs, ys = [], []
        for x in xrange(x_from, x_to):
            xs.append(x)
            ys.append(a*x + b)
        plot(xs, ys, 'g', linewidth=2)

    def set_fontsize(self, size):
        map (lambda x: x.set_fontsize(size), axes().get_xticklabels())
        map (lambda x: x.set_fontsize(size), axes().get_yticklabels())
    
    def save(self, path):
        savefig(path)
    
    def close(self):
        close()


    @staticmethod
    def bar_graph(name_value_dict, graph_title='', output_name='bargraph.png'):
        figure(figsize=(8, 8)) # image dimensions  
        title(graph_title, size='xx-small')
            
        # add bars
        for i, key in zip(range(len(name_value_dict)), name_value_dict.keys()):
            bar(i + 0.25, name_value_dict[key], color='red')
   
        # axis setup
        xticks(arange(.65, len(name_value_dict)),
               [('%s: %d' % (name, value)) for name, value in
                zip(name_value_dict.keys(), name_value_dict.values())],
               size='xx-small')
        max_value = max(name_value_dict.values())
        tick_range = arange(0, max_value, (max_value / 7))
        yticks(tick_range, size='xx-small')
        formatter = FixedFormatter([str(x) for x in tick_range])
        gca().yaxis.set_major_formatter(formatter)
        gca().yaxis.grid(which='major')
        
        savefig(output_name)
