from c_modules.ffc import _ffc
from database import *

from decimal import Decimal, getcontext
import sys

from aws import s3
from scipy.stats import chi2 as s_chi2


def sci_str(d):
    exp = int(abs(d).log10())
    if int(d / Decimal(10)**exp) == 0:
        exp += cmp(exp, 0)
    return '%.4fe%d' % (d / Decimal(10)**exp, exp)


def round2(f):
    return sci_str(f)
    s = str(f)
    if '.' in s:
        n,d = s.split('.',1)
        s = '%s.%s' % (n, d[:4])
    return s


def gstest(lst):
    l = len(lst) - 1
    if l == 0:
        return lst[0]
    elif l == 1:
        return lst[1] - 2*lst[0]
    return lst[l] - 2*lst[l-1] + lst[l-2]


def gstest_df(base, run):
    if run == 1:
        return base-1
    return base**run - 2*base**(run-1) + base**(run-2)


# Fetch Job
jobs = Job().query(
    'SELECT * FROM job WHERE type=%s ORDER BY random()',
    ('dist',))


if not jobs:
    print 'nothing to do!'
    sys.exit()


BLOCK_SIZE = 2**14
getcontext().prec = 1024

expected = db_execute_one('SELECT MAX(T.count) FROM '+
                          '(SELECT COUNT(*) AS count FROM DistributionResult GROUP BY polynomial_id) T')[0]

for job in jobs:
    results = db_execute_one('SELECT COUNT(*) FROM DistributionResult WHERE polynomial_id=%s',
                             (job.polynomial_id,))

    if results[0] == expected:
        continue

    print results[0]
    db_execute('DELETE FROM DistributionResult WHERE polynomial_id=%s', (job.polynomial_id,))
    db_commit()

    job.status  = 'working'
    job.progress = 0
    job.save()


    poly = Polynomial().query_one('SELECT * FROM polynomial WHERE id=%s',
                                  (job.polynomial_id,))
    print '-----', poly.name, '-----'

    dist_ids = dict( (d.name, d.id) for d in
                     Distribution().query('SELECT * FROM Distribution') )


    types = {}

    print poly.name
    for base in (2, 3, 5, 7, 10):
        print '==== BASE %i ====' % base
        job.progress = base / 10.0
        job.save()

        polyb = s3.get('%s-b%i' % (poly.name, base))

        lst_stats = []
        rows  = []

        for run in range(1, 5+1):
            print '>> run %i' % run
            job.message = 'base: %i, run: %i' % (base, run)
            job.save()

            stats = _ffc.count(polyb, base, run, BLOCK_SIZE)
            lst_stats.append(stats)

            freqs = Decimal(stats[-1]['freq'])
            chi2  = Decimal(stats[-1]['chi2'])
            kdist = (abs(freqs) - 1) / (Decimal(base)**run)

            def calc_gchi2(i):
                return gstest(map(lambda x: Decimal(x[i]['chi2']), lst_stats))

            def getlps(not_ok):
                lps = Decimal(0)
                for i,it in reversed(list(enumerate(stats))):
                    lps = Decimal(it['length'])
                    if not_ok(i,it):
                        return lps
                return lps

            def test_gchi2(i, it):
                df = gstest_df(base, run)
                _chi2 = s_chi2(df)
                v = calc_gchi2(i)
                return v < Decimal(str(_chi2.ppf(0.025))) \
                    or v > Decimal(str(_chi2.ppf(0.975)))

            def test_freq(i, it):
                v = abs(Decimal(it['freq'])-1)
                return v > Decimal('0.975')

            def test_kdist(i, it):
                v = abs(Decimal(it['freq']) - 1) / (Decimal(base)**run)
                return v <= 1 / Decimal(it['length']).sqrt()

            lps_freqs = getlps(test_freq)
            lps_gchi2 = getlps(test_gchi2)
            lps_kdist = getlps(test_kdist)

            gchi2 = calc_gchi2(len(lst_stats[0])-1)

            rows.append((round2(gchi2), gstest_df(base, run), round2(lps_gchi2),
                         round2(freqs), round2(lps_freqs), round2(lps_kdist)))

            print 'freqs:', round2(abs(1-freqs))
            print 'chi2: ', round2( chi2)
            print 'gchi2:', round2(gchi2)
            print 'kdist:', round2(kdist)
            
            print 'freq_lps: ', round2(lps_freqs)
            print 'gchi2_lps:', round2(lps_gchi2)
            print 'kdist_lps:', round2(lps_kdist)


            results = { 'freqs'       : str(freqs)[:128],
                        'freqs_lps'   : str(lps_freqs),
                        'gserial'     : str(gchi2)[:128],
                        'gserial_lps' : str(lps_gchi2),
                        'kdist'       : str(kdist)[:128],
                        'kdist_lps'   : str(lps_kdist) }

            for name in results:
                DistributionResult(distribution_id=dist_ids[name],
                                   result=results[name],
                                   polynomial_id=poly.id,
                                   base=base, runlength=run).save();


        #if freqs < 0.95 or freqs > 1.05:
        #    break


    job.progress = 1
    job.status  = 'done'
    job.save()
