from roots import PolyLib
from database import *
from decimal import Decimal
from scipy.stats import chi2
from math import sqrt
from collections import defaultdict
import re


def sci_str(s):
    d = Decimal(s)
    exp = int(abs(d).log10())
    if int(d / Decimal(10)**exp) == 0:
        exp += cmp(exp, 0)
    return '%.3fe%d' % (d / Decimal(10)**exp, exp)


def texify(s):
    return re.sub(r'x\^(\d+)', r'x^{\1}', s).replace('*',r'')


outdir = 'tables'

dists = dict( (d.id, d.name) for d in Distribution().all() )
print dists

dists2tex = {
    'gserial':     r'$\nabla^2\chi^2_{s,n,b}$',
    'gserial_lps': r'$\lps{\nabla^2\chi^2}{s}$',
    'df':          r'$\mathrm{df}$',

    'freqs':     r'$\maxblock{s}{n}{b}$',
    'freqs_lps': r'$\lps{\maxblock{s}{n}{b}}{s}$',

    'wwO':  r'$ww_o$',
    'wwO_lps': r'$\lps{ww_o}{s}$',
    'wwS': r'$ww_s$',
    'wwS_lps': r'$\lps{ww_s}{s}$'
    }

TEX_HEADER_1 = r'''
\begin{table}
\centering\small
\caption{Statistics for %s: $%s$}
\begin{tabularx}{\linewidth}{|l||X|X|X|X|X|}
\hline
Base & 2 &&&&\\
\hline
Block size n & 1 & 2 & 3 & 4 & 5  \\
\hline\hline
'''

TEX_HEADER_2 = r'''
\\
\hline\hline\hline
Base & %i &&&&\\
\hline
Block size n & 1 & 2 & 3 & 4 & 5  \\
\hline\hline
'''

TEX_FOOTER = r'''
\\
\hline
\end{tabularx}
\label{%s}
\end{table}
'''

def poly2tex(name):
    tex = name.replace('-', ' ').capitalize()
    if 'Sqrt' in tex:
        tex = r'$\sqrt{%s}$' % (tex.split(' ',1)[1])
    return tex.replace('Random', 'SSP')

def not_ok_gserial(base,rl):
    c = chi2(rl == 1 and base-1 or (base**rl - 2*base**(rl-1) + base**(rl-2)))
    return lambda d: d > Decimal(str(c.ppf(0.95)))

def not_ok_freq(base,rl):
    def not_ok(d):
        return abs(d-1) > Decimal('0.05')
    return not_ok

def not_ok_ww1((runs, nval, pval), fixed=True):
    N = nval+pval
    u = fixed and (Decimal(N)/2 + 1) or (Decimal(2*pval*nval)/N) + 1
    #print 'HER', Decimal(N)/2, Decimal(2*pval*nval)/N
    r2 = ((u-1)*(u-2))/(N-1)

    r = Decimal(str(sqrt(r2)))
    Z = (runs-u)/r
    reg = Decimal('1.96')*r
    #print 'HER', reg
    #print 'HER', fixed, u-reg, runs, u+reg
    return not (u-reg < runs < u+reg)

def not_ok_ww(s):
    row = map(Decimal, s.split('|'))
    return not_ok_ww1(row, False) #or not_ok_ww1(row, True)

not_ok = {
    'gserial' : not_ok_gserial,
    'freqs'   : not_ok_freq,
    'wwO'     : not_ok_ww,
    'wwS'     : not_ok_ww,
    }



chi2_b2 = [ set() for i in range(5) ]
fail = defaultdict(lambda : defaultdict(lambda : defaultdict(lambda : set())))
all_polys = set()

buckets_group = defaultdict(lambda : defaultdict(lambda : set()))
buckets = defaultdict(lambda : defaultdict(lambda : set()))
buckets_all = defaultdict(lambda : set())
for poly in Polynomial().query('SELECT * FROM Polynomial'):
    result_list = list(DistributionResult().query(
            'SELECT * FROM DistributionResult WHERE polynomial_id=%s', (poly.id,)))
    if not result_list:
        continue

    results = {} # base -> runlength -> distribution_id -> value
    all_polys.add(poly.name)

    for result in result_list:
        # unpack result
        base,rl,did = result.base, result.runlength, result.distribution_id
        dname = dists[did]

        # create needed dict's
        results[base] = results.get(base, {})
        results[base][rl] = results[base].get(rl, {})
        results[base][rl][dname] = results[base][rl].get(dname, {})

        # fill container
        results[base][rl][dname] = result.result

    #if not 10 in results:
    #    continue
    #if not all(map(lambda x: 5 in x, (results[b] for b in results ))):
    #    continue

    print '--', poly.id, poly.name, '--'

    order = ['df', 'gserial', 'gserial_lps', 'freqs', 'freqs_lps'] #, 'wwO', 'wwO_lps'] #, 'wwS', 'wwS_lps']
    #order = ['wwO', 'wwO_lps']

    bases = {2: [], 3: [], 5:[], 7:[], 10: []}
    for term in order:
        #row = {2: [dists2tex[term]], 10: [dists2tex[term]]}
        row = defaultdict(lambda : [dists2tex[term]])
        for base in (2, 3, 5, 7, 10):
            for rl in xrange(1,5+1):
                if 'lps' in term:
                    group = poly.name.split('-')[0]
                    res = results[base][rl][term]
                    if int(res) == -1:
                        #print base, term, res
                        if not 'ww' in term:
                            buckets_all[(base, rl)].add(poly.name)
                        buckets[term[:-4]][(base, rl)].add(poly.name)
                        if term.startswith('gserial'):
                            buckets_group[group][(base, rl)].add(poly.name)
                            if base == 2:
                                chi2_b2[rl-1].add(poly.name)
                        fail[term][base][rl].add(poly)
                if term == 'df':
                    # special case (degrees of freedom)
                    dof = (rl == 1) and base-1 or (base**rl - 2*base**(rl-1) + base**(rl-2))
                    row[base].append('$%s$' % dof)
                elif term[:2] == 'ww' and not 'lps' in term:
                    #print 'HER --'
                    result = results[base][rl][term]
                    #print result
                    rejectf = not_ok[term]
                    indi = (rejectf(result) and r'\uparrow') or ''
                    txt = sci_str(result.split('|')[0])
                    row[base].append('$%s%s%s$' % (indi,txt,indi))
                else:
                    result = sci_str(results[base][rl][term])
                    #print result
                    if 'lps' in term and Decimal(result) < 0:
                        result = '\infty'
                    indi = ''
                    if term in not_ok:
                        #print term, result
                        d = Decimal(result)
                        rejectf = not_ok[term](base, rl)
                        indi = (rejectf(d) and r'\uparrow') or ''
                    row[base].append('$%s%s%s$' % (indi,result,indi))
        for base in (2, 3, 5, 7, 10):
            bases[base].append(' & '.join(row[base]))

    Pstr = poly2tex(poly.name)

    fd = file('%s/summary-%s.tex' % (outdir, poly.name), 'w')
    fd.write(TEX_HEADER_1 % (Pstr, texify(PolyLib.simplify(poly.polynomial))))
    fd.write('\\\\\n\hline\n'.join(bases[2]))
    for _b in (3, 5, 7, 10):
        fd.write(TEX_HEADER_2 % _b)
        fd.write('\\\\\n\hline\n'.join(bases[_b]))
    fd.write(TEX_FOOTER % 'summary-%s' % poly.name)
    fd.close()

for k in buckets:
    print k, len(buckets[k])


def do_latex(name, dic):
    bases = {}
    for b in [2, 3, 5, 7, 10]:
        bases[b] = [len(dic[(b,rl)])
                    for rl in range(1, 5+1)]

    return (r'''\begin{table}
\centering\small
\caption{\bucket'''+name+r'''cap}
\begin{tabularx}{\linewidth}{|l||X|X|X|X|X|}
\hline
Base / block size & 1 & 2 & 3 & 4 & 5\\
\hline
%s \\
\hline
\end{tabularx}
\label{tab:bucket'''+name+r'''cap}
\end{table}''') % '\\\\ \\hline\n'.join( str(base) + ' & ' + ' & '.join('%i' % bases[base][rl] for rl in range(0,5))
                                for base in [2, 3, 5, 7, 10])

tex = do_latex('all', buckets_all)
file('%s/buckets.tex' % outdir,'w').write(tex)


for term in ['freqs', 'gserial', 'wwO']:
    tex = do_latex(term, buckets[term])
    file('%s/buckets_%s.tex' % (outdir, term), 'w').write(tex)


for group in ['pisot', 'salem', 'sqrt', 'random']:
    tex = do_latex(group, buckets_group[group])
    file('%s/buckets_group_%s.tex' % (outdir, group), 'w').write(tex)

for i in range(5):
    print 'runlength:', i+1
    print '\n'.join(sorted(chi2_b2[i]))

def sort_key(name):
    name, num = name.split('-')
    return (name, int(num))

print 'WHAT'
no_fail = set(all_polys)
lines = defaultdict(lambda : [])
counts = defaultdict(lambda : 0)
for base in (2, 3, 5, 7, 10):
    print '== BASE', base
    for rl in (1, 2, 3, 4, 5):
        print 'BLOCK SIZE', rl, ':',
        polys = list(sorted(map(lambda x: x.name, fail['gserial_lps'][base][rl]), key=sort_key))
        print ', '.join(polys)
        lines[base].append(polys)
        for p in polys:
            counts[p] += 1
            if rl == 5 and p in no_fail:
                no_fail.remove(p)


chitex = ''
for base in (2, 3, 5, 7, 10):
    table_tex = r'''
\begin{tabularx}{\linewidth}{|lX|}
\hline
Base '''+str(base)+r''' & \\
\hline
Block size & Polynomials \\
\hline
%s \\
\hline
\end{tabularx}
''' % '\\\\\n'.join( '%i & %s' % (i+1, ', '.join(map(poly2tex, line))) for i, line in enumerate(lines[base]))

    chitex += table_tex

chitex = r'''
\begin{table}
\caption{\chifailcap}
\centering\small
%s
\label{tab:chifailcap}
\end{table}
''' % chitex

    
file('%s/chi2_failure.tex' % outdir,'w').write(chitex)

rev_counts = defaultdict(lambda : [])
for p, c in counts.items():
    rev_counts[c].append(p)

for c in range(10):
    print len(rev_counts[c]), ', '.join(sorted(rev_counts[c]))

print 'NO FAIL'
print len(no_fail), ', '.join(map(poly2tex, sorted(no_fail, key=sort_key)))
