from c_modules.ffc import _ffc
from database import *
from timings import time_func
from decimal import Decimal, getcontext
from math import sqrt, log
import sys, time, gc

from aws import s3
from scipy.stats import chi2 as s_chi2


SAVE = True


def message(m):
    print '%s: %s' % (time.time(), m)

def sci_str(d):
    if abs(d) == 0:
        return '0.0000'
    exp = int(abs(d).log10())
    if int(d / Decimal(10)**exp) == 0:
        exp += cmp(exp, 0)
    return '%.4fe%d' % (d / Decimal(10)**exp, exp)


def round2(f):
    return sci_str(f)
    s = str(f)
    if '.' in s:
        n,d = s.split('.',1)
        s = '%s.%s' % (n, d[:4])
    return s


def gstest(lst):
    l = len(lst) - 1
    if l == 0:
        return lst[0]
    elif l == 1:
        return lst[1] - 2*lst[0]
    return lst[l] - 2*lst[l-1] + lst[l-2]


def gstest_df(base, run):
    if run == 1:
        return base-1
    return base**run - 2*base**(run-1) + base**(run-2)


# Fetch Job
job = Job().query_one(
    'SELECT * FROM job WHERE type=%s AND status=%s ORDER BY random() LIMIT 1',
    ('dist', 'waiting'))


if not job:
    print 'nothing to do!'
    sys.exit(1)
if SAVE:
    job.status  = 'working'
    job.progress = 0
    job.save()


poly = Polynomial().query_one('SELECT * FROM polynomial WHERE id=%s',
                              (job.polynomial_id,))
#poly = Polynomial().query_one('SELECT * FROM polynomial WHERE name=%s',
#                              ('pisot-1',))
print '-----', poly.name, '-----'




getcontext().prec = 1024


dist_ids = dict( (d.name, d.id) for d in
                 Distribution().query('SELECT * FROM Distribution') )


types = {}

print poly.name
for base in (2, 3, 5, 7, 10):
    gc.collect()

    print '==== BASE %i ====' % base
    if base in (2,3,5,7):
        print 'SKIPPING BASE %i' % base
        continue
    job.progress = base / 10.0
    job.save()

    polyb = s3.get('%s-b%i' % (poly.name, base))
    if base != 2:
        polyb = polyb[:int( (2*3**18) * log(2)/log(base) )]
    else:
        polyb = polyb[:2*3**18]

    BLOCK_SIZE = int(len(polyb)/128000)

    lst_stats = []
    #rows  = []

    for run in range(1, 5+1):
        gc.collect()

        print '>> run %i' % run
        job.message = 'base: %i, run: %i' % (base, run)
        job.save()

        cpu_time, stats = time_func(_ffc.count)(polyb, base, run, BLOCK_SIZE)
        stats = tuple(stats)
        lst_stats.append(stats)

        print 'stats computed in: %s' % cpu_time

        freqs = Decimal(stats[-1]['freq'])
        chi2  = Decimal(stats[-1]['chi2'])
        wwO   = stats[-1]['wwO']
        wwS   = stats[-1]['wwS']
        #kdist = (abs(freqs) - 1) / (Decimal(base)**run)

        def calc_gchi2(i):
            return gstest(map(lambda x: Decimal(x[i]['chi2']), lst_stats))

        def getlps(not_ok):
            lps = -1
            for i,it in reversed(list(enumerate(stats))):
                if not_ok(i,it):
                    return Decimal(lps)
                lps = it['length']
            return Decimal(lps)

        def test_gchi2(i, it):
            df = gstest_df(base, run)
            _chi2 = s_chi2(df)
            v = calc_gchi2(i)
            return v > Decimal(str(_chi2.ppf(0.95)))

        def test_freq(i, it):
            v = abs(Decimal(it['freq'])-1)
            return v > Decimal('0.05')

        def test_kdist(i, it):
            v = abs(Decimal(it['freq']) - 1) / (Decimal(base)**run)
            return v <= 1 / Decimal(it['length']).sqrt()

        def test_ww1((runs, nval, pval), fixed=True):
            N = nval+pval
            u = fixed and (Decimal(N)/2 + 1) or (Decimal(2*pval*nval)/N) + 1
            r2 = ((u-1)*(u-2))/(N-1)

            r = Decimal(str(sqrt(r2)))
            Z = (runs-u)/r
            reg = Decimal('1.96')*r
            return not (u-reg < runs < u+reg)

        def test_ww(ww):
            def test(i, it):
                res = map(Decimal, it[ww].split('|'))
                return test_ww1(res, True) or test_ww1(res, False)
            return test

        message('counting lps..')
        message('>freq')
        lps_freqs = getlps(test_freq)
        message('>gchi2')
        lps_gchi2 = getlps(test_gchi2)
        message('>wwO')
        lps_wwO   = getlps(test_ww('wwO'))
        message('>wwS')
        lps_wwS   = getlps(test_ww('wwS'))
        message('done.')

        gchi2 = calc_gchi2(len(lst_stats[0])-1)

        #rows.append((round2(gchi2), gstest_df(base, run), round2(lps_gchi2),
        #             round2(freqs), round2(lps_freqs), round2(lps_kdist)))

        print 'freqs:', round2(abs(1-freqs))
        print 'chi2: ', round2( chi2)
        print 'gchi2:', round2(gchi2)

        print 'wwO:  ', wwO
        print 'wwS:  ', wwS


        print 'freq_lps: ', round2(lps_freqs)
        print 'gchi2_lps:', round2(lps_gchi2)
        print 'wwO_lpls: ', round2(lps_wwO)
        print 'wwS_lpls: ', round2(lps_wwS)


        results = { 'freqs'       : str(freqs)[:128],
                    'freqs_lps'   : str(lps_freqs),
                    'gserial'     : str(gchi2)[:128],
                    'gserial_lps' : str(lps_gchi2),
                    'wwO'         : wwO,
                    'wwO_lps'     : str(lps_wwO),
                    'wwS'         : wwS,
                    'wwS_lps'     : str(lps_wwS)
                    }

        ''' SAVE? '''
        if SAVE:
            for name in results:
                DistributionResult(distribution_id=dist_ids[name],
                                   result=results[name],
                                   polynomial_id=poly.id,
                                   base=base, runlength=run).save();
        else:
            print 'NOT SAVING!'
        #if freqs < 0.95 or freqs > 1.05:
        #    break

if SAVE:
    job.progress = 1
    job.status  = 'done'
    job.save()
