from sympy import Symbol, Poly, pretty, numbers

import re, gmpy


''' Global Constans '''

RE_COEF_OLD = '(-?\d+)x\^(\d+)'
RE_COEF_NEW = r'\1*x^\2'

RE_COEF = r'(-?\d+)?\*x\^(\d+)'

X = Symbol('x')


''' Helper functions for Polynomials '''
class PolyLib:
    @staticmethod
    def from_string(s):
        ''' convert old-style poly format: "1x^3-1x^1-1x^0" -> "1*x^3-1*x^1-1*x^0"  '''
        s = s.replace('**', '^')
        s = re.sub(RE_COEF_OLD, RE_COEF_NEW, s)
        d = {}
        for coef, power in re.findall(RE_COEF, s):
            d[int(power)] = len(coef) and int(coef) or 1
        return PolyLib.from_dict(d)

    @staticmethod
    def from_dict(d):
        ''' convert dict of type {power: coef} to poly '''
        d = dict(((p,), numbers.Integer(d[p])) for p in d)
        return Poly(d, X)

    @staticmethod
    def get_terms(p):
        ''' return terms as list; e.g "x^3-x-1" -> [(0,-1),(1,-1),(3,1)] '''
        return map(lambda i: (i[0][0], str(i[1])), sorted(p.as_dict().items()))

    @staticmethod
    def as_dict(p):
        ''' return terms as dict: e.g. "x^3-x-1" -> {3: 1, 1: -1, 0: -1} '''
        return dict(PolyLib.get_terms(p))

    @staticmethod
    def get_coefs(p):
        ''' return terms as list; e.g "x^3-x-1" -> [-1,-1,0,1] '''
        coefs = [0]*(p.degree+1)
        for power, coef in PolyLib.get_terms(p):
            coefs[power] = coef
        return coefs

    @staticmethod
    def simplifyp(poly):
        spoly = pretty(poly).replace('**','^').replace('\n','')
        return re.findall(r'Poly\(([^,]+), x\)', spoly)[0]

    @staticmethod
    def simplify(spoly):
        ''' simplify polynomial for pretty printing '''
        spoly = re.sub(r'(-?\d+)x\^(\d+)', r'\1*x^\2', spoly)
        spoly = pretty(Poly(
                spoly, X)).replace('**','^').replace('\n','')
        return re.findall(r'Poly\(([^,]+), x\)', spoly)[0]



    @staticmethod
    def as_func(p):
        ''' return function f, st. f(x) = p(x) '''
        return eval('lambda x: %s' % p.as_basic())


    @staticmethod
    def iszero(p):
        ''' test if poly p(x) = 0 '''
        return p.LC == 0

    @staticmethod
    def isone(p):
        ''' test if poly p(x) = 1 '''
        return p.LC == 1 and p.length == 1

    @staticmethod
    def gcd(a, b):
        ''' compute greatest common divisor of polys p and q '''
        while not PolyLib.iszero(b):
            a, b = b, a % b
        return a.as_monic()

    @staticmethod
    def kill_multiple_roots(p):
        ''' compute new poly with same roots as p; however no multiple (p must have exact/rational coefs) '''

        # g := monic(p % p')
        g = PolyLib.gcd(p, p.diff())
        
        # p has no multiple roots, return
        if PolyLib.isone(g): return p

        # p has multiple roots; q = p / g has less
        print '!! Eliminated Multiple Root'
        return kill_multiple_roots(p / g)
