import gmpy
from numpy import matrix, array, dot, linalg, identity
from roots import PolyLib

def read_polynomial(polynomial):
    ''' read polynomial from string format '''
    # extract coefficients
    poly  = PolyLib.kill_multiple_roots(PolyLib.from_string(polynomial))
    coefs = PolyLib.get_coefs(poly)
    N = poly.degree
    # compute replacement matrix
    R = matrix([[ gmpy.mpz(
        int(r == 0)*-1*int(coefs[N-c-1]) +    # -1 * coef in first row
        int(r == c or r-1 == c)                # +1 in diagonal and one row under
        ) for c in xrange(N) ] for r in xrange(N) ])
    return N, R


def picarte_inv(d, itr):
    ''' compute picarte inverse '''
    i, ii   = gmpy.mpz(1), gmpy.mpz(2)
    r, r2r  = gmpy.mpz(2), gmpy.mpz(0)
    inv     = gmpy.mpz(0)
    for _itr in xrange(itr):
        # square now so r is free
        r2r  = r ** 2
        # overview r
        r   += ii
        # 2^i x_i 
        inv *= r
        # floor ( r_i^2 / b )
        qou = r2r / d
        r   = r2r-qou*d
        # add quotient to inverse
        inv += qou
        # update i and ii
        i  <<= 1
        ii **= 2
    return inv
        

def picarte_div(n, d, prec):
    ''' multiplication using picarte inverse '''
    n *= picarte_inv(d, prec)
    noise = gmpy.mpz(2)**(prec-2)
    return n >> (noise*3)


def gupta_mittal(polynomial, prec):
    ''' compute root using gupta-mittal '''
    N, R = read_polynomial(polynomial)
    R = linalg.matrix_power(R, 2**prec)
    res = dot(R, array([1]+[0]*(N-1)) )
    #print 'n:', res[0,0]
    #print 'd:', res[0,1]
    return picarte_div(res[0,0], res[0,1], prec+2)


