import re, gc

from roots import pw, mp, PolyLib

from math import sqrt,log
from gmpy import mpf, mpz, pi


RE_COEF = re.compile(r'(-?\d+)x\^(\d+)')
MP_SOLVE_PREC = 15


def laguerres_method(pol, N, init=None, use_mp=True):

    poly = PolyLib.kill_multiple_roots(PolyLib.from_string(pol))
    p    = PolyLib.as_func(poly)
    pp   = PolyLib.as_func(poly.diff())
    ppp  = PolyLib.as_func(poly.diff().diff())

    # extract degree of polynomial
    degree = poly.degree

    # we need a starting point close by
    ref_digits = None
    if init == None and use_mp:
        _, x = mp.mp_solve(pol, 12)
        x = abs(x)
        ref_digits = x.digits(2)[:2048][:-16]
    elif init == None:
        _, x = pw.power_method(pol, 12, use_mp=False)
        x = abs(x)
        ref_digits = x.digits(2)[:2048][:-16]
    else:
        x = mpf(init)

    one = mpf(1)
    for i in xrange(N):
        if i >= 6:
            print '! lm',i

        prec = max(4096, 2*3**(i+1))
        x = x.round(prec)
        gc.collect()

        px = p(x)
        if px == 0:
            print x.digits(10)
            raise Exception("Root is not irrational!")
            break

        one = one.round(prec)
        inv = one / px

        G  = pp(x) * inv
        ppp_inv = ppp(x) * inv

        sq = ((degree-1)*((degree-1)*(G**2)-degree*ppp_inv)).sqrt()
        a = degree / (G + cmp(G,0)*sq)

        _x = x
        x -= a


    # first digits are correct?
    lm_digits = x.digits(2)[:2**MP_SOLVE_PREC]

    if ref_digits:
        assert ref_digits == lm_digits[:len(ref_digits)]

    # convergence formula true for previous iteration?
    d0 = _x.digits(2)
    d1 = x.digits(2)
    for i in xrange(len(d0)):
        if d0[i] != d1[i]: break
    print i, 2*3**(N-2)
    assert i >= 2*3**(N-2)

    # compute prec in bits
    prec = 2*3**(N-1)

    # first digits match result of MPsolve?
    mp_prec = min(prec, MP_SOLVE_PREC+1)
    _, mp_root = mp.mp_solve(pol, mp_prec)
    # mp root can have wrong sign
    if cmp(mp_root, 0) != cmp(x, 0):
        mp_root *= -1
    mp_digits = mp_root.digits(2)[:len(lm_digits)]
    assert lm_digits == mp_digits

    return prec, x
    
