import re, math

from roots import PolyLib
from gmpy import mpf
from commands import getoutput
from config import BIN_PATH


''' 
  Ga: approximate
  on: output n decimals 
  Ob: output bare
'''
UNISOLVE_ARGS = '-Ga -o%i -Ob' 
UNISOLVE_CMD  = 'printf "%%s" | %s/unisolve %s' % (BIN_PATH, UNISOLVE_ARGS)

def mp_solve(polynomial, precision=10):
    """ find root of polynomial with MPsolve (polynomial in explicit format '1x^3-1x^1-1x^0) """
    # precision in decimals
    precision = 2**precision
    decimals = math.ceil((math.log(2) / math.log(10)) * (precision))
    # call unisolve
    coefs = PolyLib.get_coefs(PolyLib.kill_multiple_roots(PolyLib.from_string(polynomial)))
    mpin  = r'dri\n0\n%s\n%s' % (len(coefs)-1, r'\n'.join(map(str, coefs)))
    roots = getoutput(UNISOLVE_CMD % (mpin, decimals)).split('\n')
    root  = 0
    # find real root with largest absolute value
    for new_root in roots:
        real, imag = new_root.split()
        new_root = float(imag) == 0 and 0 or mpf(real, precision, 10)
        if abs(new_root) > abs(root):
            root = new_root
    return precision, root

