import re

from roots import pw, mp, PolyLib

from math import sqrt,log
from gmpy import mpf, mpz, pi


RE_COEF = re.compile(r'(-?\d+)x\^(\d+)')
MP_SOLVE_PREC = 15


def newtons_method(pol, N, init=None, use_mp=True):

    poly = PolyLib.kill_multiple_roots(PolyLib.from_string(pol))
    p    = PolyLib.as_func(poly)
    pp   = PolyLib.as_func(poly.diff())

    if p(-1) == 0:
        print '-1 is a rational root!'

    # extract degree of polynomial
    degree = poly.degree

    # we need a starting point close by
    ref_digits = None
    if init == None:
        _, x = mp.mp_solve(pol, 12)
        ref_digits = x.digits(2)[:2048][:-16]
    else:
        x = mpf(init)

    rational = False
    for i in xrange(N):
        if i > 20:
            print '! nt',i
        prec = max(2048, 4*2**i)
        x = x.round(prec)

        px = p(x)

        rational = False
        if px == 0:
            rational = True

        a = px / pp(x)

        _x = x
        x -= a

    if rational:
        print x.digits()[:50]
        raise Exception("Root is not irrational!")

    # first digits are correct?
    nt_digits = x.digits(2)[:2**MP_SOLVE_PREC]

    if ref_digits != None:
        assert ref_digits == nt_digits[:len(ref_digits)]

    # compute prec in bits
    prec = 2**N

    # first digits match result of MPsolve?
    if use_mp:
        mp_prec = min(prec, MP_SOLVE_PREC+1)
        _, mp_root = mp.mp_solve(pol, mp_prec)
        mp_digits = mp_root.digits(2)[:len(nt_digits)]
        assert nt_digits == mp_digits

    return (prec, x)
    
