import matplotlib
from gmpy import mpf
import math
matplotlib.use('Agg')

# convert to and from high precision floating points (decimal)
float_to_dec = lambda f : mpf(f, 128)
dec_to_float = lambda d : float(d)
dec_zero = lambda : float_to_dec(0)

from pylab import * 
from scipy.stats import ksone as s_ksone, chi2 as s_chi2

def frequencies(freqs, length, outcomes):
    expected = float(length) / outcomes
    freqs = [ freqs[digit]/expected for digit in freqs ]
    Fp = max(freqs)
    Fn = min(freqs)
    F = abs(1-Fp) < abs(1-Fn) and Fn or Fp

    _min, _exp, _max = 0.025, 1, 1.975
    return (F, (_min, _exp, _max))


def frequencies_draw(freqss, outcomes):
    xs,ys = [], []
    for i, freqs in enumerate(freqss):
        res = frequencies(freqs, sum(freqs.values()), outcomes)
        xs.append(i)
        ys.append(res[0])
    print res[0]

    plot(xs, ys, '-+')



def chi_square(freqs, length, outcomes):
    ''' plain old chi^2 '''
    expect = float_to_dec(length) / float_to_dec(outcomes)
    chi2   = dec_zero()
    _sum = 0
    for event in freqs:
        _sum += freqs[event]
        chi2 += (freqs[event] - expect)**2
    chi2 /= expect
    _chi2 = s_chi2(outcomes-1)
    _min, _exp, _max = (float(_chi2.ppf(p)) for p in (0.025, 0.5, 0.975))
    assert _sum == length, \
        "Supplied length (%i) did not match actual length: %i" % (length, _sum)
    return dec_to_float(chi2), (_min, _exp, _max)


def chi_square_circ(freqs, length, outcomes, runlength, data):
    ''' chi^2 circular/wrapped '''
    circs = [ data[length-runlength+i: length] + data[:i] 
              for i in xrange(1,runlength) ]
    for circ in circs:
        freqs[circ] = freqs.get(circ, 0) + 1
    return chi_square(freqs, length, outcomes)


def psi_value((chi0, chi1, chi2), length, outcomes, runlength, base):
    ''' psi value aka nabla chi^2 '''
    psi_df = base**runlength - 2*(base**(runlength-1)) + base**(runlength-2)
    _chi2 = s_chi2(psi_df)
    _min, _exp, _max = (float(_chi2.ppf(p)) for p in (0.025, 0.5, 0.975))
    psi = chi2 - 2*chi1 + chi0
    #print 'chis:', chi2, chi1, chi0
    #print 'int: ', _min, _exp, _max
    #print 'psi: ', psi
    #print 'ppf: ', _chi2.cdf(psi)
    #assert False, 'DEBUG'
    return chi2 - 2*chi1 + chi0, (_min, _exp, _max)
    


def kolmogorov(freqs, length, outcomes):
    ''' one-side kolmogorov-smirnov '''
    #M   = max(freqs.values())
    #obs = map(lambda x: float(x)/M, freqs.values())
    #res = s_kstest(numpy.array(obs), lambda x: x)

    D, D_ = dec_zero(), dec_zero()
    acc, total = dec_zero(), float_to_dec(length)
    N = long(total)
    for i,digit in enumerate(sorted(freqs.keys())):
        acc += freqs[digit]
        ref  = (float_to_dec(i+1) / len(freqs))
        D    = max(D,  (acc / total) - ref)
        D_   = max(D_, ref - (acc / total))
    Kn  = dec_to_float(total.sqrt() * D)
    Kn_ = dec_to_float(total.sqrt() * D_)
    #_ksone = s_ksone(N, scale=float(sqrt(N)))

    isf = lambda p : math.sqrt(0.5 * math.log(1 / (1 - p))) - 1/(6 * total.sqrt())
    _exp, _max = (dec_to_float(isf(float_to_dec(p))) for p in (0.5, 0.95))
    _min = 0

    #print Kn, Kn_
    #print '[',_exp, ';', _max, ']'
    #assert False, 'DEBUG'
    return dec_to_float(max(Kn,Kn_)), (_min, _exp, _max)
